import java.io.BufferedReader;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.io.Reader;
import java.net.URL;
import java.net.URLConnection;
import java.nio.charset.StandardCharsets;
import java.util.ArrayList;


public class HTMLLoader {

	private URL url = null;
	private String urlString = "";
	ArrayList<String> lines = new ArrayList<String>();
	ArrayList<Posting> postings = new ArrayList<Posting>();
	ArrayList<String> ignoreHosts = new ArrayList<String>();
	ArrayList<String> excludedUsers = new ArrayList<String>();
	ArrayList<Integer> excludedPosts = new ArrayList<Integer>();
	ArrayList<Integer> includedPosts = new ArrayList<Integer>();
	String root = "";
	int threadNr;
	int firstPage;
	int lastPage;
	String title="";
	
	
	public HTMLLoader()
	{
		ReadConfig();
	}
	
	private void ReadConfig()
	{
		BufferedReader reader;
		try {
			reader = new BufferedReader(new FileReader(
					"config.txt"));
			String line = reader.readLine();
			while (line != null) {
				if (line.length()> 2)
				{
					if (line.startsWith("title:"))
					{
						title = line.substring(6).trim();
					}
					if (line.startsWith("thread:"))
					{
						threadNr = Integer.parseInt(line.substring(7).trim());
					}
					if (line.startsWith("from:"))
					{
						firstPage = Integer.parseInt(line.substring(5).trim());
					}
					if (line.startsWith("to:"))
					{
						lastPage = Integer.parseInt(line.substring(3).trim());
					}
					if (line.startsWith("-host:"))
					{
						ignoreHosts.add(line.substring(6).trim());
					}
					if (line.startsWith("-user:"))
					{
						excludedUsers.add(line.substring(6).trim());
					}
					if (line.startsWith("-post:"))
					{
						excludedPosts.add(Integer.parseInt(line.substring(6).trim()));
					}
					if (line.startsWith("+post:"))
					{
						includedPosts.add(Integer.parseInt(line.substring(6).trim()));
					}
				}
				line = reader.readLine();
			}
			reader.close();
		} catch (IOException e) {
			e.printStackTrace();
		}
	}
	
	public void Load()
	{
		
		this.root = "content/"+threadNr+"/";
		this.urlString = "http://thereminworld.com/Forums/T/"+ threadNr;
		for (int page = firstPage; page <= lastPage; page++)
		{
			LoadPage(page);
			ExtractPostings(page);
		}
		
	}
	
	public void LoadPage(int page)
	{
		new File(root+"pages").mkdirs();
		String fileUrl = root+"pages/"+(page-1)+".html";
		boolean useFile = new File(fileUrl).exists();
		lines = new ArrayList<String>();
		boolean isLastPage = page ==lastPage;
		URLConnection con;
		try {
			Reader r;
			PrintWriter pw = null;
			if (useFile)
			{
				r = new InputStreamReader(new FileInputStream(fileUrl),StandardCharsets.UTF_8);
			}
			else
			{
				if (!isLastPage)
				{
					pw = new PrintWriter(new FileWriter(fileUrl));
				}
				this.url = new URL(urlString  + "?Page=" + (page-1));
				con = this.url.openConnection();
				r = new InputStreamReader(con.getInputStream(), StandardCharsets.UTF_8);
			}
				
			StringBuilder buf = new StringBuilder();
			while (true) {
			  int ch = r.read();
			  if (ch < 0)
			    break;
			  char c = (char) ch;
			  buf.append(c);
			  if (c=='\n')
			  {
				  lines.add(buf.toString());
				  if (!useFile && !isLastPage)
				  {
					  pw.print(buf.toString());
				  }
				  buf = new StringBuilder();
			  }
			}
			lines.add(buf.toString());
			  if (!useFile && !isLastPage)
			  {
				pw.print(buf.toString());
				pw.close();
			  }
		} catch (IOException e) {
			e.printStackTrace();
		}
		System.out.println("Loaded page " + page + " of " + lastPage);
	}
	
	
	private void CreatePosting(ArrayList<String> postingLines, int pageNr)
	{
		boolean excluded = false;
		Posting posting = new Posting(root, threadNr, postingLines, ignoreHosts, pageNr);
		for (String user:excludedUsers)
		{
			if (user.equalsIgnoreCase(posting.username.trim()))
			{
				excluded = true;
			}
		}
		for (int post:excludedPosts)
		{
			if (post == posting.postSeqNr)
			{
				excluded = true;
			}
		}
		for (int post:includedPosts)
		{
			if (post == posting.postSeqNr)
			{
				excluded = false;
			}
		}
		if (!excluded)
		{
			postings.add(posting);
		}

	}
	
	public void ExtractPostings(int pageNr)
	{
		ArrayList<String> postingLines = new ArrayList<String>();
		boolean beginFound = false;
		boolean endFound = false;
		boolean isFirst = true;
		for (String line: lines)
		{
			beginFound = false;
			if (line.trim().startsWith("<div class=\"forum-post\">"))
			{
				beginFound = true;
			}
			if (beginFound && line.trim().startsWith("<div class=\"forums-control-bar row\">"))
			{
				endFound = true;
			}
			
			if (beginFound || endFound)
			{
				if (!isFirst)
				{
					CreatePosting(postingLines, pageNr);
				
				}
				postingLines = new ArrayList<String>();
				isFirst = false;
			}
			
			postingLines.add(line);
		}
		if (postingLines.size()>0)
		{
			CreatePosting(postingLines, pageNr);
		}
	}
	
	private void ReadTemplate(PrintWriter pw, String filename)
	{
		BufferedReader reader;
		try {
			reader = new BufferedReader(new FileReader(filename));
			String line = reader.readLine();
			while (line != null) {
				if (line.contains("TITLE"))
				{
					line = line.replaceAll("TITLE", title);
				}
				pw.println(line);
				line = reader.readLine();
			}
			reader.close();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public void Export()
	{
		PrintWriter pw;
		try {
			
			pw = new PrintWriter(new FileWriter(title + ".html"));
			ReadTemplate(pw, "header.html"); 
			
			for (Posting posting: postings)
			{
				pw.print(posting);
			}
			ReadTemplate(pw, "footer.html"); 
			
			pw.close();

		} catch (IOException e) {
			e.printStackTrace();
		}

	}


	
}
